<?xml version="1.0" encoding="US-ASCII"?>
<?xml-stylesheet type="text/xsl" href="../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                version="2.0">

<xs:doc info="$Id: deepEqual.xsl,v 1.3 2008/11/18 19:45:32 gkholman Exp $"
        filename="deepEqual.xsl" vocabulary="DocBook">
  <xs:title>Crude difference report between two XML instances</xs:title>
  <para>
    A XSLT stylesheet to crudely report differences between two
    XML instances by reporting the apexes of differences based on child
    position of the parent.  This code does not attempt to line up
    children of two parents when absent children throw the count of children
    off.
  </para>
<programlisting>
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
</programlisting>
</xs:doc>

<xs:output>
  <para>There is no main output, so simply use text</para>
</xs:output>
<xsl:output method="text"/>

<xs:param name="other" ignore-ns="yes">
  <para>The root node of the other document.</para>
  <para>
    Note that when using Saxon, this is specified by prefixing the argument
    name with "<literal>+</literal>" as in:
  </para>
  <blockquote>
    <para><literal>"+other=t:\file\name\here.xml"</literal></para>
  </blockquote>
</xs:param>
<xsl:param name="other" as="document-node()" required="yes"/>

<xs:template>
  <para>
    Start at the beginning, only delving deep if there are differences.
  </para>
</xs:template>
<xsl:template match="/">
  <!--build up a message so that there is a single error exit with
      multiple reports-->
  <xsl:variable name="reports">
    <!--only bother going deep if there is a difference somewhere-->
    <xsl:variable name="this" select="."/>
    <xsl:variable name="count"
                  select="max( (count($this/node()), count($other/node())) )"/>
    <!--check if any are different-->
    <xsl:if test="some $p in 1 to $count
             satisfies not(deep-equal( $this/node()[$p],$other/node()[$p] ) )">
      <!--there are, so begin acting on child nodes-->
      <xsl:for-each select="1 to $count">
        <xsl:if test="not( deep-equal( $this/node()[current()], 
                                       $other/node()[current()] ) )">
          <xsl:apply-templates select="$this/node()[current()]">
            <xsl:with-param name="other" select="$other/node()[current()]"/>
          </xsl:apply-templates>
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
  </xsl:variable>
  <!--check to see if there were any problems-->
  <xsl:if test="string($reports)">
    <xsl:message terminate="yes" select="$reports"/>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>Check the children of each node or differences.</para>
  <para>Without children, this node must be in error.</para>
  <xs:param name="other">
    <para>Corresponding node in the other tree.</para>
  </xs:param>
</xs:template>
<xsl:template match="node()">
  <xsl:param name="other" as="node()?"/>
  <xsl:variable name="this" select="."/>
  <xsl:variable name="count"
                select="max( (count($this/node()), count($other/node())) )"/>
  <!--check for any individual differences-->
  <xsl:choose>
    <xsl:when test="some $p in 1 to $count 
                   satisfies deep-equal( $this/node()[$p],$other/node()[$p] )">
      <!--as long as some are equal, keep looking down until all are unequal-->
      <xsl:for-each select="1 to $count">
        <xsl:if test="not( deep-equal( $this/node()[current()], 
                                       $other/node()[current()] ) )">
          <xsl:apply-templates select="$this/node()[current()]">
            <xsl:with-param name="other" select="$other/node()[current()]"/>
          </xsl:apply-templates>
        </xsl:if>
        <!--if there are no nodes at this point for the first tree, then
            the second tree must have excess children-->
        <xsl:if test="not( $this/node()[current()] )">
          <xsl:apply-templates mode="report" 
                               select="$other/node()[current()]"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <!--there are no children that are equal; don't bother going further-->
      <xsl:apply-templates mode="report" select="."/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xs:template>
  <para>Report a node in error.</para>
</xs:template>
<xsl:template mode="report" match="node()">
  <xsl:for-each select="ancestor-or-self::*">
    <xsl:text/>/<xsl:value-of select="name(.)"/>
    <xsl:if test="../..">
      <!--only report siblings below the document element-->
      <xsl:text/>[<xsl:number/>]<xsl:text/>
    </xsl:if>
  </xsl:for-each>
  <xsl:choose>
    <xsl:when test="self::text()">/text()[<xsl:number/>]</xsl:when>
    <xsl:when test="self::comment()">/comment()[<xsl:number/>]</xsl:when>
    <xsl:when test="self::processing-instruction()"
                           >/processing-instruction()[<xsl:number/>]</xsl:when>
    <xsl:when test="not(self::*)">/@<xsl:value-of select="name(.)"/></xsl:when>
  </xsl:choose>
  <xsl:text>
</xsl:text>
</xsl:template>

</xsl:stylesheet>