<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0">

<!--$Id: addprefix.xsl,v 1.2 2003/07/26 22:25:36 G. Ken Holman Exp $-->
<!--Change all elements in the default namespace to the specified prefix.
-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:param name="prefix" select="''"/>

<xsl:variable name="default-ns-uri" 
              select="*/namespace::*[name(.)='']"/>

<xsl:template match="/">
  <xsl:if test="not($prefix)">
    <xsl:message terminate="yes">
      <xsl:text>Must supply prefix= for old default namespace</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:if test="not($default-ns-uri)">
    <xsl:message terminate="yes">
   <xsl:text>Document element does not utilize the default namespace</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:apply-templates select="node()"/>
</xsl:template>

<!--copy all nodes not interested in being changed-->
<xsl:template match="comment()|processing-instruction()|@*|text()">
  <xsl:call-template name="check-name"/>
  <xsl:copy/>
</xsl:template>

<!--recreate element node of interest or just copy-->
<xsl:template match="*">
  <xsl:call-template name="check-name"/>
  <xsl:choose>
    <xsl:when test="namespace-uri(.) = $default-ns-uri">
      <xsl:element name="{$prefix}:{name(.)}" namespace="{$default-ns-uri}">
        <xsl:apply-templates select="@*|node()"/>
      </xsl:element>
    </xsl:when>
    <xsl:otherwise>
      <xsl:copy>
        <xsl:apply-templates select="@*|node()"/>
      </xsl:copy>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--check a node's name for pre-existing use of the supplied prefix-->
<xsl:template name="check-name">
  <xsl:if test="substring-before( name(.),':' )=$prefix">
    <xsl:message terminate="yes">
      <xsl:text>Found pre-existing use of supplied prefix:</xsl:text>
      <xsl:for-each select="ancestor-or-self::*"><!--show context-->
        <xsl:text>/</xsl:text>
        <xsl:if test="not(contains(name(.),':'))">
          <xsl:value-of select="$prefix"/>
        </xsl:if>
        <xsl:value-of select="name(.)"/>
        <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
          <xsl:text/>[<xsl:number/>]<xsl:text/>
        </xsl:if>
      </xsl:for-each>
    </xsl:message>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>
