<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<!--$Id: checkuse.xsl,v 1.5 2003/07/26 22:25:36 G. Ken Holman Exp $-->
<!--check for improper use of Literate XSLT(TM) attributes in no namespace-->

<!--note that if the input instance does have an attribute of the same
    local name as a Literate XSLT attribute, create in importing stylesheet
    that accepts the use of all such attributes for all appropriate elements-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:output method="text"/><!--there is no output, only error messages-->

<xsl:template match="/">
  <xsl:apply-templates select="*"/>
  <xsl:apply-templates select="*">
    <xsl:with-param name="error" select="'yes'"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="*"><!--check all attributes of all elements-->
  <xsl:param name="error"/>
  <xsl:apply-templates select="*|@*">
    <xsl:with-param name="error" select="$error"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="xsl:*"><!--assume XSLT will catch all these attributes-->
  <xsl:param name="error"/>
  <xsl:apply-templates select="*">
    <xsl:with-param name="error" select="$error"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="@*"/><!--if not caught, then not a problem-->

<!--list all the Literate XSLT  attributes here-->
<xsl:template match="
  @apply-templates     |
  @call-template       |
  @content-apply       |
  @content-call        |
  @content-copy        |
  @content-mode        |
  @content-name        |
  @content-infix       |
  @content-prefix      |
  @content-suffix      |
  @content-value       |
  @content-variable    |
  @content-verbatim    |
  @comment             |
  @infix               |
  @keep                |
  @keep                |
  @lose                |
  @lose                |
  @match               |
  @mode                |
  @name                |
  @order               |
  @prefix              |
  @priority            |
  @see                 |
  @set                 |
  @set-comment         |
  @suffix              |
  @use
">
  <xsl:param name="error"/>
  <xsl:choose>
    <xsl:when test="$error">
      <xsl:message terminate="yes">
        <xsl:text>Ambiguities found ... exiting.</xsl:text>
      </xsl:message>
    </xsl:when>
    <xsl:otherwise>
      <xsl:message>
        <xsl:text>Ambiguity found: 
</xsl:text>
        <xsl:for-each select="ancestor-or-self::*"><!--show context-->
          <xsl:text>/</xsl:text>
          <xsl:value-of select="name(.)"/>
          <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
            <xsl:text/>[<xsl:number/>]<xsl:text/>
          </xsl:if>
        </xsl:for-each>
        <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
