<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:x="http://www.CraneSoftwrights.com/ns/literate-xslt"
  xmlns:a="http://www.CraneSoftwrights.com/ns/literate-xslt/attribute"
  version="1.0">

<!--$Id: key2html.xsl,v 1.15 2003/10/23 12:36:57 G. Ken Holman Exp $-->
<!--This produces a three-column report indicating the key value, the
    XPath address to the key value, and the value that was replaced by
    the key value (values of attributes in non-null namespaces are not
    preserved).  It is assumed that the instance has been produced by
    the keyxml.xsl stylesheet from a representative data file.

    Parameters:

       values=yes (to expose the value column)
       repeat=yes (to distinguish like sibling elements)
 -->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:param name="values"/><!--set to reveal sample values found in instance-->
<xsl:param name="repeat"/><!--used to differentiate sibling XPath addresses-->

<xsl:key name="xpaths" match="*" use="@x:path"/>
<xsl:key name="apaths" use="concat(../@x:path,'/@',name(.))" match="@*[
namespace-uri(.)!='http://www.CraneSoftwrights.com/ns/literate-xslt'
and
namespace-uri(.)!='http://www.CraneSoftwrights.com/ns/literate-xslt/attribute'
]"/>

<xsl:template match="/">
  <html>
    <title>
      <xsl:if test="/*/@x:label">
        <xsl:value-of select="/*/@x:label"/> - <xsl:text/>
      </xsl:if>
      <xsl:text>Keyed instance</xsl:text>
    </title>
    <body>
      <xsl:if test="/*/@x:label">
        <p><xsl:value-of select="/*/@x:label"/></p>
      </xsl:if>
      <table border="1">
        <xsl:call-template name="do-rows"/>
      </table>
    </body>
  </html>
</xsl:template>

<xsl:template name="do-rows">
  <xsl:choose>
    <xsl:when test="$repeat">
      <xsl:for-each select="//*[ text()[normalize-space(.)] ] | //@*[
namespace-uri(.)!='http://www.CraneSoftwrights.com/ns/literate-xslt'
and
namespace-uri(.)!='http://www.CraneSoftwrights.com/ns/literate-xslt/attribute'
]">
        <xsl:call-template name="do-row"/>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="//*[ generate-id(.)=
                                 generate-id(key('xpaths',@x:path)) ]
                               [ text()[normalize-space(.)] ]|
                            //@*[ generate-id(.)=
                                  generate-id(key('apaths',
                                  concat(../@x:path,'/@',name(.))))]">
        <xsl:call-template name="do-row"/>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="do-row">
  <tr>
    <td nowrap="nowrap">
      <a name="key-{translate( ., '!', '' )}">
        <small>
          <xsl:value-of select="translate( ., '!', '' )"/>
         <xsl:if test="not( contains( ., '.' ) )">&#xa0;&#xa0;</xsl:if>
        </small>
      </a>
    </td>
    <!--report the path-->
    <td>
        <xsl:variable name="this">
          <xsl:choose>
            <xsl:when test="self::*">
              <xsl:value-of select="name(.)"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text/>@<xsl:value-of select="name(.)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable name="path">
          <xsl:choose>
            <xsl:when test="self::*">
              <xsl:call-template name="do-path"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:for-each select="..">
                <xsl:call-template name="do-path">
                  <xsl:with-param name="suffix" select="'/'"/>
                </xsl:call-template>
              </xsl:for-each>
              <samp style="white-space:nowrap">
                <xsl:text/>@<xsl:value-of select="name(.)"/>
              </samp>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <small onclick="clipboardData.setData( 'Text',&quot;{translate($path,'!&#x200b; ','')}&quot; );window.status=&quot;{$this} written!&quot;;return true"
          onMouseOver="window.status=&quot;{$this}&quot;;return true;"
          onMouseOut="window.status='';return true">
          <xsl:copy-of select="$path"/>
        </small>
    </td>
    <xsl:if test="$values">
      <!--get the value that was preserved in the instance-->
      <td>
        <small>
          <xsl:choose>
            <xsl:when test="self::*">
              <xsl:value-of select="@x:value"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="../@a:*[local-name(.)=name(current())]"/>
            </xsl:otherwise>
          </xsl:choose>
        </small>
      </td>
    </xsl:if>
  </tr>
</xsl:template>

<!--the path is displayed with a zero-width space after each "/" in order to
    allow the path to wrap in the table cell in a well-behaved fashion-->
<xsl:template name="do-path">
  <xsl:param name="suffix"/>
  <xsl:param name="path" select="@x:path"/>
  <xsl:choose>
    <xsl:when test="contains( $path, '/' )">
      <samp style="white-space:nowrap">
        <xsl:value-of select="substring-before( $path, '/' )"/>/<xsl:text/>
      </samp>
      <xsl:call-template name="do-path">
        <xsl:with-param name="path" select="substring-after( $path, '/' )"/>
        <xsl:with-param name="suffix" select="$suffix"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <samp style="white-space:nowrap">
        <xsl:value-of select="$path"/>
        <xsl:value-of select="$suffix"/>
      </samp>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
