<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:x="http://www.CraneSoftwrights.com/ns/literate-xslt"
  xmlns:a="http://www.CraneSoftwrights.com/ns/literate-xslt/attribute"
  version="1.0">

<!--$Id: keyxml.xsl,v 1.7 2003/10/18 15:08:17 G. Ken Holman Exp $-->
<!--Fill an XML instance with path and ordinal information.

    Adds path information for each element in x:path.
    Copies null-namespace-attribute nodes to the a: namespace.
    Replaces non-white-space-only text nodes with "!xx!"
    Replaces attribute nodes with "!xx-aa!"
-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:param name="prefix" select="''"/>
<xsl:param name="prefix-x" select="'x'"/>
<xsl:param name="prefix-a" select="'a'"/>

<xsl:param name="repeat"/><!--used to signal distinction in sibling elements-->
<xsl:param name="label"/><!--used to identify the source document in reports-->

<!--copy all non-element nodes not interested in being reported-->
<xsl:template match="comment()|processing-instruction()|text()">
  <xsl:copy/>
</xsl:template>

<!--preserve path information about the node-->
<xsl:template name="add-path-name">
    <xsl:attribute name="{$prefix-x}:path"
      namespace="http://www.CraneSoftwrights.com/ns/literate-xslt">

      <xsl:for-each select="ancestor-or-self::*"><!--show context-->
        <xsl:text>/</xsl:text>
        <xsl:if test="not(contains(name(.),':')) and $prefix">
          <xsl:value-of select="$prefix"/>:<xsl:text/>
        </xsl:if>
        <xsl:value-of select="name(.)"/>
        <xsl:if test="$repeat and
                      preceding-sibling::*[name(.)=name(current())]">
          <xsl:text/>[<xsl:number/>]<xsl:text/>
        </xsl:if>
      </xsl:for-each>
    </xsl:attribute>
</xsl:template>

<!--copy all element nodes, keeping the path information-->
<xsl:template match="*">
  <xsl:copy>
    <xsl:if test="not(parent::*) and $label">
      <xsl:attribute name="{$prefix-x}:label"
                  namespace="http://www.CraneSoftwrights.com/ns/literate-xslt">
        <xsl:value-of select="$label"/>
      </xsl:attribute>
    </xsl:if>
    <xsl:call-template name="add-path-name"/>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

<!--elements with at least one non-white-space-only child text node-->
<xsl:template match="*[text()[normalize-space(.)]]">
  <xsl:copy>
    <xsl:apply-templates select="@*"/>
    <xsl:call-template name="add-path-name"/>

                      <!--preserve value of all child text nodes-->
    <xsl:attribute name="{$prefix-x}:value"
      namespace="http://www.CraneSoftwrights.com/ns/literate-xslt">
      <xsl:for-each select="text()">
        <xsl:value-of select="."/>
      </xsl:for-each>
    </xsl:attribute>

    <xsl:apply-templates select="node()"/><!--process children-->
  </xsl:copy>
</xsl:template>

<!--copy all attributes and replace with ordinal-->
<xsl:template match="@*">
  <xsl:attribute name="{$prefix-a}:{local-name(.)}"
        namespace="http://www.CraneSoftwrights.com/ns/literate-xslt/attribute">
    <xsl:value-of select="."/>
  </xsl:attribute>
  <xsl:choose>
    <xsl:when test="namespace-uri(.)=''">
      <xsl:attribute name="{name(.)}">
        <xsl:text>!</xsl:text>
        <xsl:for-each select="..">
          <xsl:number level="any" count="*"/>
        </xsl:for-each>.<xsl:value-of select="position()"/>
        <xsl:text>!</xsl:text>
      </xsl:attribute>
    </xsl:when>
    <xsl:otherwise>
      <xsl:attribute name="{name(.)}" namespace="namespace-uri(.)">
        <xsl:text>!</xsl:text>
        <xsl:for-each select="..">
          <xsl:number level="any" count="*"/>
        </xsl:for-each>-<xsl:value-of select="position()"/>
        <xsl:text>!</xsl:text>
      </xsl:attribute>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!--replace all non-white-space-only text nodes with ordinal-->
<xsl:template match="text()[normalize-space(.)]">
  <xsl:text>!</xsl:text>
  <xsl:for-each select="..">
    <xsl:number level="any" count="*"/>
  </xsl:for-each>
  <xsl:text>!</xsl:text>
</xsl:template>

</xsl:stylesheet>
