<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:x="http://www.CraneSoftwrights.com/ns/literate-xslt"
                exclude-result-prefixes="x"
                version="1.0">

<!--$Id: lit2xpath.xsl,v 1.4 2003/07/30 17:13:32 G. Ken Holman Exp $-->
<!--Find all of the literate attributes using XPath expressions-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:output indent="yes"/>

<xsl:key name="verbatims" match="x:verbatim" use="@label"/>

<xsl:template match="/">
  <result>
    <xsl:apply-templates select="*"/>
  </result>
</xsl:template>

<xsl:template match="xsl:apply-templates">
  <xsl:param name="path"/>

  <apply-templates>
    <xsl:call-template name="next">
      <xsl:with-param name="path" select="$path"/>
      <xsl:with-param name="this" select="@select"/>
      <xsl:with-param name="stop" select="'yes'"/>
    </xsl:call-template>
  </apply-templates>
</xsl:template>

<xsl:template match="xsl:value-of">
  <xsl:param name="path"/>

  <value-of>
    <xsl:call-template name="next">
      <xsl:with-param name="path" select="$path"/>
      <xsl:with-param name="this" select="@select"/>
      <xsl:with-param name="stop" select="'yes'"/>
    </xsl:call-template>
  </value-of>
</xsl:template>

<xsl:template match="xsl:for-each">
  <xsl:param name="path"/>
  
  <for-each>
    <xsl:call-template name="next">
      <xsl:with-param name="path" select="$path"/>
      <xsl:with-param name="this" select="@select"/>
    </xsl:call-template>
  </for-each>
</xsl:template>

<xsl:template match="*">
  <xsl:param name="path"/>

  <xsl:choose>
    <xsl:when test="@x:apply-templates">
      <apply-templates>
        <xsl:call-template name="next">
          <xsl:with-param name="path" select="$path"/>
          <xsl:with-param name="this" select="@x:apply-templates"/>
        </xsl:call-template>
      </apply-templates>
    </xsl:when>
    <xsl:when test="@x:match">
      <match>
        <xsl:call-template name="next">
          <xsl:with-param name="path" select="$path"/>
          <xsl:with-param name="this" select="@x:match"/>
        </xsl:call-template>
      </match>
    </xsl:when>
    <xsl:when test="@x:for-each">
      <for-each>
        <xsl:call-template name="next">
          <xsl:with-param name="path" select="$path"/>
          <xsl:with-param name="this" select="@x:for-each"/>
        </xsl:call-template>
      </for-each>
    </xsl:when>
    <xsl:when test="not( @x:see )">
      <xsl:call-template name="next">
        <xsl:with-param name="path" select="$path"/>
        <xsl:with-param name="this" select="''"/>
      </xsl:call-template>
    </xsl:when>

  </xsl:choose>

  <!--
  <xsl:for-each select="@x:prefix | @x:suffix">
    <xsl:apply-templates select="key( 'verbatims', . )/node()">
      <xsl:with-param name="path" select="'/'"/>
    </xsl:apply-templates>
  </xsl:for-each>

  <xsl:for-each select="@x:infix">
    <xsl:apply-templates select="key( 'verbatims', . )/node()">
      <xsl:with-param name="path" select="$path"/>
    </xsl:apply-templates>
  </xsl:for-each>
  -->

</xsl:template>

<xsl:template name="next">
  <xsl:param name="path"/>
  <xsl:param name="this"/>
  <xsl:param name="stop"/>

  <xsl:variable name="use-path">
    <xsl:choose>
      <xsl:when test="substring($this,1,1)='/'"/>
      <xsl:otherwise>
        <xsl:value-of select="$path"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="use-this">
    <xsl:choose>
      <xsl:when test="$path='/'">
        <xsl:value-of select="$this"/>
      </xsl:when>
      <xsl:when test="substring($this,1,1)='/'">
        <xsl:value-of select="$this"/>
      </xsl:when>
      <xsl:when test="$this">
        <xsl:value-of select="concat( '/', $this )"/>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>

  <xsl:if test="$this">
    <xsl:attribute name="path">
      <xsl:value-of select="concat( $use-path, $use-this )"/>
    </xsl:attribute>
    <xsl:attribute name="this">
      <xsl:value-of select="$this"/>
    </xsl:attribute>
    <xsl:call-template name="loc-attribute"/>
  </xsl:if>

  <xsl:choose>
    <xsl:when test="$stop"/>
    <xsl:when test="@x:content-value">
      <value-of>
        <xsl:call-template name="next">
          <xsl:with-param name="path" select="concat($use-path,$use-this)"/>
          <xsl:with-param name="this" select="@x:content-value"/>
          <xsl:with-param name="stop" select="'yes'"/>
        </xsl:call-template>
      </value-of>
    </xsl:when>
    <xsl:when test="@x:content-copy">
      <copy-of>
        <xsl:call-template name="next">
          <xsl:with-param name="path" select="concat($use-path,$use-this)"/>
          <xsl:with-param name="this" select="@x:content-copy"/>
          <xsl:with-param name="stop" select="'yes'"/>
        </xsl:call-template>
      </copy-of>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates>
        <xsl:with-param name="path" select="concat($use-path,$use-this)"/>
      </xsl:apply-templates>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>

<xsl:template name="loc-attribute">
    <xsl:attribute name="loc">
      <xsl:for-each select="ancestor-or-self::*">
        <xsl:text>/</xsl:text>
        <xsl:value-of select="name(.)"/>
        <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
          <xsl:text>[</xsl:text>
          <xsl:number/>
          <xsl:text>]</xsl:text>
        </xsl:if>
      </xsl:for-each>
    </xsl:attribute>
</xsl:template>

<!--reserve explicit for no processing-->
<xsl:template match="x:replace | x:verbatim"/>

<xsl:template match="text()|*|comment()|processing-instruction()" priority="-1"/>

</xsl:stylesheet>
