<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<!--$Id: xml4lit.xsl,v 1.5 2003/07/26 22:25:37 G. Ken Holman Exp $-->
<!--Check the file named in the parameter $xmlfile for all paths found
    in the source file description of paths-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:output method="text"/>

<xsl:param name="xmlfile"/>
<xsl:param name="noerrors"/>

<xsl:variable name="checkargs"><!--command line checking only-->
  <xsl:if test="not($xmlfile)">
    <xsl:message terminate="yes">
      <xsl:text>Missing name of XML file in xmlfile= parameter</xsl:text>
    </xsl:message>
  </xsl:if>
</xsl:variable>

<xsl:variable name="debug"/><!--set this non-empty to give status messages-->

<!--send all the results to error and exit with error-->
<xsl:template match="/paths">
  <xsl:variable name="result"><!--put the resultof processing into a string-->
    <xsl:apply-templates select="e"/>
  </xsl:variable>
  <xsl:if test="string( $result )"><!--an empty result is no errors-->
    <xsl:choose>
      <xsl:when test="$noerrors"><!--having a result is not considered error-->
        <xsl:value-of select="string( $result )"/>
      </xsl:when>
      <xsl:otherwise><!--a result is considered an error-->
        <xsl:message terminate="yes">
          <xsl:text>
</xsl:text>
          <xsl:value-of select="string( $result )"/>
        </xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>

<!--handle an element construct and process  attributes and element children-->
<xsl:template match="e">
  <xsl:param name="xml" select="document( $xmlfile, . )/*"/>

  <xsl:if test="$debug"><!--give status-->
    <xsl:message>
      <xsl:text>Local: </xsl:text>
      <xsl:value-of select="@n"/>
      <xsl:text>, XML: </xsl:text>
      <xsl:value-of select="name( $xml )"/>
    </xsl:message>
  </xsl:if>

  <xsl:if test="@n != '*' and name( $xml ) != @n"><!--internal problem-->
  <xsl:message>Unexpected!  <xsl:value-of select="name($xml)"/>
  <xsl:text> : </xsl:text>
  <xsl:value-of select="@n"/>
  </xsl:message>
  </xsl:if>

  <xsl:for-each select="a"><!--process each of the attributes-->
    <xsl:if test="$debug">
      <xsl:message>Checking @<xsl:value-of select="@n"/></xsl:message>
    </xsl:if>
    <xsl:if test="not( $xml/attribute::*[local-name(.)=current()/@n] )">
      <xsl:call-template name="mismatch"/><!--attribute not found-->
    </xsl:if>
  </xsl:for-each>

  <xsl:for-each select="e"><!--checking children-->
    <xsl:choose>
      <xsl:when test="@n='*'"><!--go through a child that is only a wildcard-->
        <xsl:choose>
          <xsl:when test="not( $xml/* )"><!--at tree bottom;no more elements-->
            <xsl:call-template name="mismatch"/>
          </xsl:when>
          <xsl:otherwise><!--process with all next level children-->
            <xsl:apply-templates select=".">
              <xsl:with-param name="xml" select="$xml/*"/>
            </xsl:apply-templates>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise><!--explicitly named children-->
        <xsl:variable name="nextxml" 
                      select="$xml/*[name(.)=current()/@n ][1]"/>
        <xsl:choose><!--cannot find child in next level of children-->
          <xsl:when test="not( $nextxml )">
            <xsl:call-template name="mismatch"/>
          </xsl:when>
          <xsl:otherwise><!--process with named child-->
            <xsl:apply-templates select=".">
              <xsl:with-param name="xml" select="$nextxml"/>
            </xsl:apply-templates>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>

<!--add to the result string a message about which Literate XSLT pattern
    wasn't found-->
<xsl:template name="mismatch">
  <xsl:for-each select="ancestor::e"><!--report full path down to error-->
    <xsl:text>/</xsl:text>
    <xsl:value-of select="@n"/>
  </xsl:for-each>
  <xsl:choose><!--last node is the element or attribute in question-->
    <xsl:when test="self::e">
      <xsl:text>/</xsl:text>
      <xsl:value-of select="@n"/>
    </xsl:when>
    <xsl:when test="self::a">
      <xsl:text>/@</xsl:text>
      <xsl:value-of select="@n"/>
    </xsl:when>
  </xsl:choose>
  <xsl:text> at </xsl:text>
  <xsl:value-of select="@loc"/>
  <xsl:text>
</xsl:text>
</xsl:template>

<xsl:template match="text()"/>

</xsl:stylesheet>
