<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<!--$Id: xpathtxt.xsl,v 1.2 2003/07/26 22:25:37 G. Ken Holman Exp $-->
<!--Exposes the XPath addresses of all terminal nodes of an instance-->
<!--note that text nodes are normalized which may mean they are discarded
    if they contain meaningful whitespace-->

<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-lxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<xsl:output method="text"/>

<xsl:param name="prefix"/><!--optional prefix in place of default namespace-->

<xsl:template match="*[* or text()[normalize-space(.)]]">
  <xsl:apply-templates select="*|@*|text()"/><!--not a terminal-->
</xsl:template>

<xsl:template match="*|@*|text()" mode="force"><!--is a meaningful terminal-->
  <xsl:call-template name="force"/>
</xsl:template>

<xsl:template name="force" 
              match="*|@*|text()[normalize-space(.)]" ><!--may be a terminal-->
  <xsl:for-each select="ancestor-or-self::*"><!--show context-->
    <xsl:text>/</xsl:text>
    <xsl:if test="$prefix">
      <xsl:value-of select="concat( $prefix, ':' )"/>
    </xsl:if>
    <xsl:value-of select="name(.)"/>
    <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
      <xsl:text/>[<xsl:number/>]<xsl:text/>
    </xsl:if>
  </xsl:for-each>
  <xsl:choose>
    <xsl:when test="text()[normalize-space(.)]"><!--non-empty text node-->
      <xsl:text>
</xsl:text>
      <xsl:apply-templates select="*|@*|text()" mode="force"/>
    </xsl:when>
    <xsl:when test="self::* and not(*)"><!--empty element-->
      <xsl:text>
</xsl:text>
      <xsl:apply-templates select="@*|text()" mode="force"/>
    </xsl:when>
    <xsl:when test="self::*"><!--non-empty-element-->
      <xsl:text>
</xsl:text>
      <xsl:apply-templates select="*|@*|text()"/>
    </xsl:when>
    <xsl:when test="self::text()"><!--a text node; forced or non-empty-->
      <xsl:text> > </xsl:text>
      <xsl:value-of select="normalize-space(.)"/>
      <xsl:text>
</xsl:text>
    </xsl:when>
    <xsl:otherwise><!--an attribute node-->
      <xsl:value-of select="concat('/@',name(.),'=',.)"/>
      <xsl:text>
</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="text()"/><!--terminal but not meaningful-->

</xsl:stylesheet>
